//
//  SNGBenExternalAddConfirmVC.swift
//  Nextgen Dev
//
//  Created by Cuong on 5/21/20.
//  Copyright © 2020 Chinh.nd. All rights reserved.
//

import UIKit

class SNGBenExternalAddConfirmVC: SNGBaseViewController {
    @IBOutlet weak var header: SNGHead!
    @IBOutlet weak var colBenDetail: UICollectionView!
    @IBOutlet weak var btnConfirm: UIButton!
    @IBOutlet weak var btnCancel: UIButton!
    
    var detail: SNGBeneficiaryModel?
    var listDataCell : [SNGComboboxModel] = []
    var lblTitle : String?
    let service = SNGBenModel()
    let benType : String = "EXTERNAL"
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.registerCell()
        
        self.btnConfirm.setTitle("Global.Confirm".localized(), for: .normal)
        self.btnCancel.setTitle("Global.Cancel".localized(), for: .normal)
        self.header.lbTitle.text = self.lblTitle
        
        self.header.selectBack = {
            self.backViewController()
        }
    }
    
    func registerCell() {
        self.colBenDetail.register(UINib(nibName: SNGCollectionViewCellDetail.className, bundle: nil), forCellWithReuseIdentifier: SNGCollectionViewCellDetail.className)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        self.createData()
    }
    
    func createData() {
        if detail != nil {
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.bankname".localized(), text: detail!.bankName))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.provinceName".localized(), text: detail!.provinceName))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.branchName.detail".localized(), text: detail!.branchName))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.accountNo".localized(), text: detail!.toAccount))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.benName".localized(), text: detail!.benName))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.billing.hint".localized(), text: detail!.hintName))
    
           
            self.colBenDetail.reloadData()
        }
    }
    
    @IBAction func onConfirmClick(_ sender: Any) {
        
        self.detail!.benType = self.benType
        self.service.addBen(temp: self.detail!)
        self.service.completionSuccess = {
            [weak self] message in
            guard let `self` = self else { return }
            if message == "ADD_SUCCESS" {
                DispatchQueue.main.async {
                    let continueAction = Action(with: "Global.Continue".localized(), style: .normalRed) {[weak self] in
                        guard let `self` = self else { return }
                        self.navigationController?.popViewController(animated: true)
                    }
                    
                    let closeAction = Action(with: "Global.Exit".localized(), style: .normalGray) {[weak self] in
                        guard let `self` = self else { return }
                        self.backToViewController(SNGBenExternalListVC.self)
                    }
                    
                    self.showPopupTemplate(title: "settings.menu.ben.add.success".localized(), content:  "settings.menu.ben.add.successContent".localized(), inputs: [], actions: [continueAction,closeAction], axis: .vertical, style: AlertStyle.normal, showImage: true, alertImageType: TypeImage.SUCCESS)
                }
            }
        }
        
        self.service.completionFailed = {
            [weak self] message in
            guard let `self` = self else { return }
           
                DispatchQueue.main.async {
                    self.showError(error: message)
                }
        }
    }
    
    @IBAction func onCancelClick(_ sender: Any) {
        self.backToViewController(SNGBenExternalAddVC.self, reload: true)
    }
}

extension SNGBenExternalAddConfirmVC: UICollectionViewDataSource, UICollectionViewDelegate {
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        
        return listDataCell.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: SNGCollectionViewCellDetail.className, for: indexPath) as! SNGCollectionViewCellDetail
        
        cell.key.text = self.listDataCell[indexPath.row].id
        cell.value.text = self.listDataCell[indexPath.row].text
        if(indexPath.row == self.listDataCell.count){
            cell.separator.isHidden = true
        }
        return cell
    }
}

// set style
extension SNGBenExternalAddConfirmVC: UICollectionViewDelegateFlowLayout {
    func collectionView(_ collectionView: UICollectionView,
                        layout collectionViewLayout: UICollectionViewLayout,
                        minimumLineSpacingForSectionAt section: Int) -> CGFloat {
        return 0
    }
}
