//
//  SNGBenExternalDetailVC.swift
//  Nextgen Dev
//
//  Created by Cuong on 5/20/20.
//  Copyright © 2020 Chinh.nd. All rights reserved.
//

import UIKit

class SNGBenExternalDetailVC: SNGBaseViewController {
    @IBOutlet weak var header: SNGHead!
    @IBOutlet weak var colTranDetail: UICollectionView!
    @IBOutlet weak var btnDelete: UIButton!
    @IBOutlet weak var btnEdit: UIButton!
    
    var detail: SNGBeneficiaryModel?
    var listDataCell : [SNGComboboxModel] = []
    
    let service = SNGBenModel()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.registerCell()
        loadData()
        
        self.btnDelete.setTitle("settings.menu.ben.detail.btn.delete".localized(), for: .normal)
        self.btnEdit.setTitle("settings.menu.ben.detail.btn.edit".localized(), for: .normal)
        self.header.lbTitle.text = "settings.menu.ben.outside.detail.title".localized()
        
        self.header.selectBack = {
            self.backViewController()
        }
    }
    func loadData(){
        self.service.completionSuccess = {
            message in
            DispatchQueue.main.async {
                if message == "DELETE_SUCCESS" {
                    //Di chuyển đến màn hình danh sách
                    self.showPopUpWarning(viewControll: self, title: "settings.menu.ben.delete.success".localized(), content: "settings.menu.ben.delete.successContent".localized(),buttonText: "Global.Agree".localized(), type: TypeShowImagePopup.IMAGE, typeImage: TypeImage.SUCCESS, successHandler: {
                        self.backViewController()
                    })
                }
            }
        }
        
        self.service.completionFailed = {
            message in
            DispatchQueue.main.async {
                self.showError(error: message)
        }
    }
    }
    override func viewWillAppear(_ animated: Bool) {
        self.createData()
    }
    func registerCell() {
         self.colTranDetail.register(UINib(nibName: SNGCollectionViewCellDetail.className, bundle: nil), forCellWithReuseIdentifier: SNGCollectionViewCellDetail.className)
    }
    
    func createData() {
        listDataCell = [];
        if detail != nil {
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.bankname".localized(), text: detail!.bankName))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.provinceName".localized(), text: detail!.provinceName))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.branchName.detail".localized(), text: detail!.branchName))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.accountNo".localized(), text: detail!.toAccount.inserting(every: 4, with: " ")))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.outside.benName".localized(), text: detail!.benName))
            listDataCell.append(SNGComboboxModel(id : "settings.menu.ben.billing.hint".localized(), text: detail!.hintName))
     
            
            self.colTranDetail.reloadData()
        }
    }
    
    @IBAction func onDeleteClick(_ sender: Any) {
        // Hiển thị confirm dialog xóa
        let agreeAction = Action(with: "Global.Agree".localized(), style: .normalRed) {[weak self] in
            self?.handleDeleteConfirmation()
        }
        let closeAction = Action(with: "Global.Cancel".localized(), style: .normalGray) {}
        self.showPopupTemplate(title: "settings.menu.ben.delete.title".localized(), content: "settings.menu.ben.delete.notifiContent".localized(), actions: [agreeAction, closeAction], axis: .vertical)
    }
    
    private func handleDeleteConfirmation() {
        self.service.deleteBen(benId: self.detail!.id)
        
    }
    
    @IBAction func onEditClick(_ sender: Any) {
        let storyboard = UIStoryboard(name: "SNGBenExternalEdit", bundle: nil)
            let viewController = storyboard.instantiateViewController(withIdentifier: "SNGBenExternalEditVC") as! SNGBenExternalEditVC
            viewController.detail = self.detail
            viewController.hidesBottomBarWhenPushed = true
            self.navigationController?.pushViewController(viewController, animated: true)
    }
}


extension SNGBenExternalDetailVC: UICollectionViewDelegate {}

extension SNGBenExternalDetailVC: UICollectionViewDataSource {
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
       
        return listDataCell.count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: SNGCollectionViewCellDetail.className, for: indexPath) as! SNGCollectionViewCellDetail
        cell.key.text = self.listDataCell[indexPath.row].id
        cell.value.text = self.listDataCell[indexPath.row].text
        if(indexPath.row == self.listDataCell.count){
            cell.separator.isHidden = true
        }
        return cell
    }
}

// set style
extension SNGBenExternalDetailVC: UICollectionViewDelegateFlowLayout {
    func collectionView(_ collectionView: UICollectionView,
                        layout collectionViewLayout: UICollectionViewLayout,
                        minimumLineSpacingForSectionAt section: Int) -> CGFloat {
        return 0
    }
}
