//
//  SNGBenExternalEditVC.swift
//  Nextgen Dev
//
//  Created by Cuong on 5/20/20.
//  Copyright © 2020 Chinh.nd. All rights reserved.
//

import UIKit

class SNGBenExternalEditVC: SNGBaseViewController {
    
    @IBOutlet weak var header: SNGHead!
    @IBOutlet weak var lblBankName: SNGLabel!
    @IBOutlet weak var cbBank: SNGComboboxTemp!
    @IBOutlet weak var lblAcountNo: SNGLabel!
    @IBOutlet weak var txtAccountNo: UITextField!
    @IBOutlet weak var lblBenName: SNGLabel!
    @IBOutlet weak var txtBenName: UITextField!
    @IBOutlet weak var lblProvinceName: SNGLabel!
    @IBOutlet weak var cbProvinceName: SNGComboboxTemp!
    @IBOutlet weak var lblBranchName: SNGLabel!
    @IBOutlet weak var cbBranchName: SNGComboboxTemp!
    @IBOutlet weak var btnExecute: SNGButton!
    @IBOutlet weak var txtHintName: UITextField!
    @IBOutlet weak var lblHintName: SNGLabel!
    
    let validator = Validator()
    let commonModel = SNGCommonModel()
    
    var bankCode : String = ""
    var bankName : String = ""
    var cityId : String = ""
    var cityName : String = ""
    var branchCode : String = ""
    var branchName : String = ""
    var detail : SNGBeneficiaryModel?
    var branchUrl : String = "/services/sibcobeneficiary/api/common/branches-bank?"
    let service = SNGBenModel()
    override func viewDidLoad() {
        super.viewDidLoad()
        self.createData()
      
        self.cbBank.input.setStylePlaceholder(font: Utils.buildFontFrom(type: "MTR15"), text: "settings.template.placeholder.out.bankname".localized(), color: UIColor.black)
        self.cbProvinceName.input.setStylePlaceholder(font: Utils.buildFontFrom(type: "MTR15"), text: "settings.template.placeholder.out.provincename".localized(), color: UIColor.black)
        self.cbBranchName.input.setStylePlaceholder(font: Utils.buildFontFrom(type: "MTR15"), text: "settings.template.placeholder.out.branchname".localized(), color: UIColor.black)
        lblBankName.text = "settings.ben.add.bankname".localized()
          lblAcountNo.text = "settings.ben.add.accountNo".localized()
          txtAccountNo.placeholder = "settings.ben.add.accountNo.input".localized()
          lblBenName.text = "settings.ben.add.benname".localized()
          txtBenName.placeholder = "settings.ben.add.benname.input".localized()
            lblProvinceName.text = "settings.menu.ben.outside.provinceName.add".localized()
            lblBranchName.text = "settings.menu.ben.outside.branchName".localized()
        btnExecute.setTitle("settings.ben.add.save".localized(), for: .normal)
        self.header.lbTitle.text = "settings.menu.ben.outside.add.title".localized()
        self.header.selectBack = {
            self.backViewController()
        }
        
        // Disable execute button by default
        self.btnExecute.setDisable()
        self.addEventTouchInView()
    }
  
    
    func createData() {
        // setup bank select box
        self.cbBank.title = "settings.template.placeholder.out.bankname".localized()
        
        self.cbBank.url = "/services/sibcobeneficiary/api/common/banks"
        self.cbBank.viewControll = self
        self.cbBank.valueFiledName = "bankCode"
        self.cbBank.textFiledName = "bankName"
        self.cbBank.selectSuccess = self.onBankSelected
        
        // setup province select box
        self.cbProvinceName.title = "settings.template.placeholder.out.provincename".localized()
        
        self.cbProvinceName.url = "/services/sibcobeneficiary/api/common/cities"
        self.cbProvinceName.viewControll = self
        self.cbProvinceName.valueFiledName = "cityID"
        self.cbProvinceName.textFiledName = "cityName"
        self.cbProvinceName.selectSuccess = self.onProvinceSelected
        
        // set branch select box
        self.cbBranchName.title = "settings.template.placeholder.out.branchname".localized()
        
        if self.bankCode == "" {
            self.cbBranchName.url = ""
        } else {
            self.cbBranchName.url = self.branchUrl + "bankCode=" + self.bankCode + "&cityID=" + self.cityId
        }
        self.cbBranchName.viewControll = self
        self.cbBranchName.valueFiledName = "branchCode"
        self.cbBranchName.textFiledName = "branchName"
        self.cbBranchName.selectSuccess = self.onBranchSelected
        
        self.txtAccountNo.delegate = self
        self.txtBenName.delegate = self
        
        self.commonModel.completionSuccess = {
            [weak self] message in
            guard let `self` = self else { return }
            if message == "ACCOUNT_INFO_SUCCESS" {
                DispatchQueue.main.async {
                    let bankAccountInfo = self.commonModel.bankAccountInfo
                    if bankAccountInfo.shortName != "" {
                        self.detail?.activeNapas = true
                        // set model
                        self.branchCode = bankAccountInfo.branchCode
                        self.cityId = bankAccountInfo.cityID
                        self.txtBenName.text = bankAccountInfo.shortName.uppercased()
                      
                        // set combox
                        self.cbBranchName.input.text = bankAccountInfo.branchName
                        self.cbProvinceName.input.text = bankAccountInfo.cityName
                        
                        // Disable other combo
                        self.cbBranchName.setDisable()
                        self.cbProvinceName.setDisable()
                        self.txtBenName.setDisable()
                        
                        // Enable Execute button
                        self.btnExecute.setEnable()
                    } else {
                        // Enabel other combo
                        self.branchCode = ""
                        self.cityId = ""
                        self.txtBenName.text = ""
                        self.txtHintName.text = ""
                        // set combox
                        self.cbBranchName.input.text = ""
                        self.cbProvinceName.input.text = ""
                        self.detail?.activeNapas = false
                        self.cbBranchName.setEnable()
                        self.cbProvinceName.setEnable()
                        self.txtBenName.setEnable()
                        // Disable Execute button
                        if self.cbProvinceName.input.text!.isEmpty && self.cbBranchName.input.text!.isEmpty {
                            self.btnExecute.setDisable()
                        }
                    }
                }
            }
        }
        
        self.commonModel.completionFailed = { [weak self] message in
            guard let `self` = self else { return }
            DispatchQueue.main.async {
                self.showError(error: message)
            }
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.header.lbTitle.text = "settings.menu.ben.outside.edit.title".localized()
        self.txtBenName.text = self.detail!.benName
        self.txtAccountNo.text = self.detail!.toAccount.inserting()
        self.cbBank.input.text = self.detail!.bankName
        self.cbBranchName.input.text = self.detail!.branchName
        self.cbProvinceName.input.text = self.detail!.provinceName
        self.bankCode = self.detail!.bankCode
        self.branchCode = self.detail!.branchCode
        self.cityId = self.detail!.provinceCode
        self.bankName = self.detail!.bankName
        self.txtHintName.text = self.detail!.hintName
        //            self.cbProvinceName.setEnable()
        //            self.cbBranchName.setEnable()
        self.btnExecute.setEnable()
        self.cbBranchName.message = nil
        self.cbBranchName.url = self.branchUrl + "bankCode=" + self.bankCode + "&cityID=" + self.cityId
//        self.commonModel.getBankAccountInfo(accountNum: self.detail!.toAccount.replaceSpace(), type: "EXTERNAL", bankCode: self.bankCode)
    }
    
    // Handle select bank
    func onBankSelected(selected: SNGComboboxModel) {
        let bank_code = self.bankCode
        self.bankCode = selected.id
        self.bankName = selected.text
        self.cbBank.input.text = selected.text
        
        if self.bankCode == "" {
            self.cbBranchName.message =  "settings.menu.ben.popup.warining".localized()
        } else {
            self.cbBranchName.message = nil
            self.cbBranchName.url = self.branchUrl + "bankCode=" + self.bankCode + "&cityID=" + self.cityId
            
            self.txtAccountNo.text = ""
            self.txtBenName.text = ""
            self.cbBranchName.input.text = ""
            self.cbProvinceName.input.text = ""
            self.txtBenName.setEnable()
            self.cbProvinceName.setEnable()
            self.cbBranchName.setEnable()
        }
        
        if (bank_code != selected.id) {
            self.branchCode = ""
            self.branchName = ""
            self.cbProvinceName.input.text = ""
            self.cbBranchName.input.text = ""
            
            self.cbProvinceName.setEnable()
            self.cbBranchName.setEnable()
        }
        
        if self.bankCode != "" && self.cityId != "" && self.branchCode != "" && self.txtAccountNo.text != "" && self.txtBenName.text != "" {
            self.btnExecute.setEnable()
        } else {
            self.btnExecute.setDisable()
        }
    }
    
    // Handle select province
    func onProvinceSelected(selected: SNGComboboxModel) {
        let city_code = self.cityId
        self.cityId = selected.id
        self.cityName = selected.text
        self.cbProvinceName.input.text = selected.text
        
        if self.bankCode == "" {
            self.cbBranchName.message =  "settings.menu.ben.popup.warining".localized()
        } else {
            self.cbBranchName.message = nil
            self.cbBranchName.url = self.branchUrl + "bankCode=" + self.bankCode + "&cityID=" + self.cityId
        }
        
        if city_code != selected.id {
            self.branchCode = ""
            self.branchName = ""
            self.cbBranchName.input.text = nil
        }
        
        if self.bankCode != "" && self.cityId != "" && self.branchCode != "" && self.txtAccountNo.text != "" && self.txtBenName.text != "" {
            self.btnExecute.setEnable()
        } else {
            self.btnExecute.setDisable()
        }
    }
    
    // Handle select branch
    func onBranchSelected(selected: SNGComboboxModel) {
        self.branchCode = selected.id
        self.branchName = selected.text
        self.cbBranchName.input.text = selected.text
        
        if self.bankCode != "" && self.cityId != "" && self.branchCode != "" && self.txtAccountNo.text != "" && self.txtBenName.text != "" {
            self.btnExecute.setEnable()
        } else {
            self.btnExecute.setDisable()
        }
    }
    
    // Make sure that all required fields are filled
    private func validateForm() -> Bool {
        if self.bankCode == "" {
            return false
        } else {
            return true
        }
    }
    
    // Handle form submit
    @IBAction func onExecuteClick(_ sender: Any) {
        if validateForm() {
            detail!.bankCode = self.bankCode
            detail!.bankName = self.bankName
            detail!.provinceCode = self.cityId
            detail!.provinceName = self.cbProvinceName.input.text!
            detail!.branchCode = self.branchCode
            detail!.branchName = self.cbBranchName.input.text!
            detail!.toAccount = self.txtAccountNo.text!.replaceSpace()
            detail!.benName = self.txtBenName.text!.uppercased()
            self.btnExecute.setEnable()
            self.closeKeyboard()
            let continueAction = Action(with: "Global.Agree".localized(), style: .normalRed) {[weak self] in
                guard let `self` = self else { return }
                self.service.updateBen(temp: self.detail!)
            }
            let closeAction = Action(with: "Global.Cancel".localized(), style: .normalGray) {[weak self] in
                guard let `self` = self else { return }
                self.backViewController()
            }
            self.showPopupTemplate(title: "settings.menu.ben.delete.edit".localized(), content:  "settings.menu.ben.delete.editContent".localized(), inputs: [], actions: [continueAction,closeAction], axis: .vertical, style: AlertStyle.normal)
            
            self.service.completionSuccess = { [weak self] message in
                guard let `self` = self else { return }
                if message == "UPDATE_SUCCESS" {
                    DispatchQueue.main.async {
                        
                        self.showPopUpWarning(viewControll: self, title: "settings.menu.ben.edit.success.title".localized(), content: "settings.menu.ben.edit.success".localized(),buttonText: "Global.Agree".localized(), type: TypeShowImagePopup.IMAGE, typeImage: TypeImage.SUCCESS, successHandler: {
                            DispatchQueue.main.async {
                                for controller in self.navigationController!.viewControllers as Array {
                                    if controller.isKind(of: SNGBenExternalDetailVC.self) {
                                        let view = controller as! SNGBenExternalDetailVC
                                        view.detail = self.detail
                                        self.navigationController!.popToViewController(view, animated: true)
                                        break
                                    }
                                }

                            }
                        })
                    }
                }
            }
            
            self.service.completionFailed = { [weak self] message in
                guard let `self` = self else { return }

                DispatchQueue.main.async {
                    self.showError(error: message)
                }
            }
        } 
    }
    @objc func closeKeyboard(){
        self.view.endEditing(true)
    }
}

extension SNGBenExternalEditVC: UITextFieldDelegate {
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        self.closeKeyBoard()
        switch textField {
        case self.txtAccountNo:
            if self.txtAccountNo.text == "" {
                self.txtAccountNo.updateFocusIfNeeded()
                self.txtBenName.setEnable()
                self.cbBranchName.setEnable()
                self.cbProvinceName.setEnable()
                self.cbProvinceName.input.text = ""
                self.cbBranchName.input.text = ""
                
            } else {
                self.txtAccountNo.text = self.txtAccountNo.text!.stringByRemovingVietNamese()
                let numberAcc = self.txtAccountNo.text!.replaceSpace()
                self.txtAccountNo.text = self.txtAccountNo.text!.inserting()
                if self.bankCode != "" {
                    self.commonModel.getBankAccountInfo(accountNum: numberAcc, type: "EXTERNAL", bankCode: self.bankCode)
                }
            }
            break
        case self.txtBenName:
            self.txtBenName.text = self.txtBenName.text!.stringByRemovingVietNamese().uppercased()
        default:
            break
        }
        
        // Check if all fields are filled, enable execute button
        if self.bankCode != "" && self.cityId != "" && self.branchCode != "" && self.txtAccountNo.text != "" && self.txtBenName.text != "" {
            self.btnExecute.setEnable()
        } else {
            self.btnExecute.setDisable()
        }
    }
    
    func changeText(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String){
        if string.count != 1 {
            textField.text = (textField.text! as NSString).replacingCharacters(in:range, with: string.trim())
            DispatchQueue.main.async(execute: {
                let newPosition = textField.position(from:textField.beginningOfDocument, offset:range.location+string.trim().count)
                if newPosition != nil {
                    textField.selectedTextRange = textField.textRange(from: newPosition!, to: newPosition!)
                }
            })
        } else {
            textField.text = (textField.text! as NSString).replacingCharacters(in:range, with: string)
            DispatchQueue.main.async(execute: {
                let newPosition = textField.position(from: textField.beginningOfDocument, offset: range.location + string.count)
                if newPosition != nil {
                    textField.selectedTextRange = textField.textRange(from: newPosition!, to: newPosition!)
                }
            })
        }
        
    }
    
    // Prevent the user to enter illegal characters and over max length
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        
        guard let textFieldText = textField.text,
            let rangeOfTextToReplace = Range(range, in: textFieldText) else {
                return false
        }
        
        let substringToReplace = textFieldText[rangeOfTextToReplace]
        let count = textFieldText.count - substringToReplace.count + string.count
        
        if textField == self.txtAccountNo {
            self.txtBenName.text = ""
            if string != "" {
                let textNotSpace  = (textField.text! as NSString).replacingCharacters(in:range, with: string.trim()).replaceSpace()
                if (textNotSpace.count <= 35 && validator.validate(text: string, with: [.accountFormat])){
                    changeText(textField, shouldChangeCharactersIn: range,replacementString: string)
                }
                return false;
            }
        } else if textField == self.txtBenName {
            if string != "" {
                return count <= 70 && validator.validate(text: string, with: [.toAccountName])
            }
        } else if textField == self.txtHintName {
            if string != "" {
                if count > 35 {
                    return false
                }
            }
        }
        
        return true
    }
    
   
    
}
