//
//  SNGBenExternalListVC.swift
//  Nextgen Dev
//
//  Created by Cuong on 5/19/20.
//  Copyright © 2020 Chinh.nd. All rights reserved.
//

import UIKit

class SNGBenExternalListVC: SNGBaseViewController {
    @IBOutlet weak var header: SNGHead!
    @IBOutlet weak var inputSearch: SNGInputSearch!
    @IBOutlet weak var colTemplates: UICollectionView!
    @IBOutlet weak var viewAdd: UIView!
    
    let service = SNGBenModel()
    var resultCount: Int = 0
    var orgData = [SNGBeneficiaryModel]()
    var isCheckVC = true
    override func viewDidLoad() {
        super.viewDidLoad()
        self.registerCell()

        inputSearch.txtSearch.placeholder = "Global.Search".localized()
        
        self.colTemplates.contentInset = UIEdgeInsets(top: 16, left: 0, bottom: 16, right: 0)
        
        if isCheckVC {
            header.selectBack = {
                self.backViewController()
            }
            viewAdd.isHidden = false
        } else {
            header.selectBack = {
                self.dismiss(animated: true, completion: nil)
            }
            viewAdd.isHidden = true
        }
        
       self.dismissKey()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.createData()
        orgData = [SNGBeneficiaryModel]()
    }
    
 
    func registerCell() {
        self.colTemplates.register(UINib(nibName: SNGBeneficiaryItem.className, bundle: nil),
                                forCellWithReuseIdentifier: SNGBeneficiaryItem.className)
        
        //add target search
        self.inputSearch.txtSearch.addTarget(self, action: #selector(doSearch(_:)), for: .editingChanged)
    }

    func createData() {
        self.header.lbTitle.text = "settings.menu.ben.outside.list".localized()
        self.service.getListBeneficiaries(benType: "EXTERNAL")
        self.service.completionSuccess = {
            [weak self] message in
            guard let `self` = self else { return }
            if message == "SUCCESS" {
                DispatchQueue.main.async {
                    self.resultCount = self.service.beneficiaries.count
                    self.orgData = self.service.beneficiaries
                    self.colTemplates.reloadData()
                }
            }
        }
        self.service.completionFailed = { message in
            DispatchQueue.main.async {
                self.showError(error: message)
            }
        }
    }
    
    @objc func doSearch(_ textField: UITextField) {
        let keySearch = textField.text!.lowercased().trimmingCharacters(in: .whitespacesAndNewlines)
        
        let filtered = self.orgData.filter({$0.benName.lowercased().contains(keySearch) || $0.toAccount.lowercased().contains(keySearch)})
        
        self.service.beneficiaries = filtered
        
        if keySearch == "" {
            self.service.beneficiaries = self.orgData
        }
        
        self.resultCount = self.service.beneficiaries.count
        self.colTemplates.reloadData()
    }
    
    @IBAction func onAddClick(_ sender: Any) {
        let storyboard = UIStoryboard(name: "SNGBenExternalAdd", bundle: nil)
        let viewController = storyboard.instantiateViewController(withIdentifier: "SNGBenExternalAddVC") as! SNGBenExternalAddVC
        viewController.isCreate = true
        viewController.hidesBottomBarWhenPushed = true
        self.navigationController?.pushViewController(viewController, animated: true)
    }
}

extension SNGBenExternalListVC: UICollectionViewDataSource {
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        if (self.resultCount == 0) {
            self.colTemplates.setEmptyMessage("settings.menu.ben.empty".localized())
        } else {
            self.colTemplates.restore()
        }
        return resultCount
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: SNGBeneficiaryItem.className, for: indexPath) as! SNGBeneficiaryItem
        
        if self.service.beneficiaries.count == 0 {
            return cell
        }
        
        cell.title.text = self.service.beneficiaries[indexPath.row].toAccount.inserting(every: 4, with: " ")
        cell.title.textColor = Global.redColor
        cell.desc.text =  self.service.beneficiaries[indexPath.row].benName
        cell.icon.image = convertBase64ToImage(imageString: self.service.beneficiaries[indexPath.row].bankLogo)
        cell.toBenStyle()
        
        return cell
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        if isCheckVC {
            let storyboard = UIStoryboard(name: "SNGBenExternalDetail", bundle: nil)
            let viewController = storyboard.instantiateViewController(withIdentifier: "SNGBenExternalDetailVC") as! SNGBenExternalDetailVC
            
            viewController.detail = self.service.beneficiaries[indexPath.row]
            
            viewController.hidesBottomBarWhenPushed = true
            self.navigationController?.pushViewController(viewController, animated: true)
        } else {
            dismiss(animated: true) {
            }
        }
    }
}

// set style
extension SNGBenExternalListVC: UICollectionViewDelegateFlowLayout {
    func collectionView(_ collectionView: UICollectionView,
                        layout collectionViewLayout: UICollectionViewLayout,
                        sizeForItemAt indexPath: IndexPath) -> CGSize {
        return CGSize(width: Global.screenW - 32, height: 60)
    }
    
    func collectionView(_ collectionView: UICollectionView,
                        layout collectionViewLayout: UICollectionViewLayout,
                        minimumLineSpacingForSectionAt section: Int) -> CGFloat {
        return 10.0
    }
}

